# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
import GridUtils
from GenericAI import GenericAI

class AmazonsGameState(GameState):
    def moves_from_position(self, pos):
        for mdir in GridUtils.direction_array(self.grid.ndim):
            # look for valid moves only
            for mpos,mval in GridUtils.enumerate_values_with_direction(self.grid, pos, mdir):
                if mval==0:
                    yield mpos
                else: # hit edge or blocker on move, go to next direction
                    break

    def moves_and_arrows_from_position(self, pos):
        for mpos in self.moves_from_position(pos):
            # look for valid arrow shots (can shoot back to starting position)
            for adir in GridUtils.direction_array(self.grid.ndim):
                for apos,aval in GridUtils.enumerate_values_with_direction(self.grid, mpos, adir):
                    if aval==0 or apos==pos:
                        yield (mpos, apos)
                    else:
                        break
                    
    def successors(self):
        for pos in self.enumerate_coords_with_value(self.current_player):
            for mpos,apos in self.moves_and_arrows_from_position(pos):
                # move to mpos and shoot arrow (barrier) to apos
                nextgrid = self.grid.copy()
                nextgrid[pos]  = 0
                nextgrid[mpos] = self.current_player
                nextgrid[apos] = -1
                yield ((pos, mpos, apos), self.next_state_with_grid(nextgrid))
                
    def has_scores(self):
        return False

    def winning_player(self):
        return self.previous_player()
        
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # arrow moves and grows, animation starts when the player piece stops
        moveanim  = PGPieceAnimation.move_animation(move[0], move[1])
        arrowanim = PGPieceAnimation.new_piece_animation(move[1], -1, start_time=moveanim.end_time())
        arrowanim.segments[0].to_position = move[2]
        return PGPieceAnimation.AnimationGroup([moveanim, arrowanim])

class AmazonsAI (GenericAI):
    def game_utility(self, gstate):
        # look at total number of moves available, counting possible arrows would be more accurate but is too slow
        mymoves  = [len(list(gstate.moves_from_position(pos))) for pos in gstate.enumerate_coords_with_value(gstate.current_player)]
        oppmoves = [len(list(gstate.moves_from_position(pos))) for pos in gstate.enumerate_coords_with_value(gstate.next_player())]
        # could give extra utility for a piece being completely trapped
        return sum(mymoves) - sum(oppmoves)
        
